/**
	\file
	\brief Functions for reading and writing read-all files.

	The read-all file contains simplified result of photometry for
	a set of frames. Use cmpack-readall object to make a read-all file 
	from a set of photometry files. You can pass it to the cmpack-list 
	object to make an output table.

	This file format was inherited from the varfind	project and because 
	it not extensible without breaking the compatibility it's planed to 
	be replaced in C-Munipack version 2. Please, do not use this functions 
	in newly written code.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_allfile.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_ALLFILE_H_INCLUDED
#define _CMPACK_ALLFILE_H_INCLUDED

#include "cmpack_table.h"

/***************    Private data structures    *************************/

/** 
	\brief Read-all file.
	\details This private structure holds the content of the read-all file.
*/
typedef struct _CmpackAllFile CmpackAllFile;

/********************   Public functions   **************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Create a new read-all file
	\details This function makes an empty read-all file and returns 
	pointer to it. 
	\return pointer a new reference to the file
*/
CmpackAllFile *cmpack_all_init(void);

/**
	\brief Load the read-all file.
	\param[out]	file		new file context
	\param[in] 	filename	path + file name
	\param[in] 	flags 		reserved, use always zero
	\return zero on success, error code on failure
*/
int cmpack_all_load(CmpackAllFile **file, const char *filename, CmpackLoadMode flags);
	
/**
	\brief Save the read-all file.
	\param[in]	file		file context
	\param[in] 	filename	path + file name
	\return zero on success, error code on failure
*/
int cmpack_all_save(CmpackAllFile *file, const char *filename);
	
/**
	\brief Copy a file
	\details Makes a copy of source file to the destination file.
	All content of the destination file will be deleted.
	\param[in] dstfile		destination file context
	\param[in] srcfile		source file context
	\return zero on success, error code on failure
*/
int cmpack_all_copy(CmpackAllFile *dstfile, const CmpackAllFile *srcfile);

/**
	\brief Clear all data
	\param[in]	file		file context
*/
void cmpack_all_clear(CmpackAllFile *file);

/** 
	\brief Test if given file seems to be a readall file
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_all_test(const char *filename);

/** 
	\brief Test if the content of the buffer might be a readall file
	\details The function checks the data in the buffer and returns nonzero,
	when the buffer contain first 'buflen' bytes from a readall file.
	This function is used for file format autodetection.
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_all_test_buffer(const char *buffer, int buflen, int filesize);


/******************   File header definition   *******************************/

/**
	\brief Set aperture identifier
	\param[in] file			file context
	\param[in] aperture		aperture identifier
*/
void cmpack_all_set_aperture(CmpackAllFile *file, int aperture);

/**
	\brief Get aperture identifier
	\param[in] file			file context
	\return aperture identifier or zero on failure
*/
int cmpack_all_get_aperture(CmpackAllFile *file);

/**
	\brief Set filter name
	\param[in] file			file context
	\param[in] filter		filter name
*/
void cmpack_all_set_filter(CmpackAllFile *file, const char *filter);

/**
	\brief Get filter name
	\param[in] file			file context
	\return pointer to internal memory buffer or NULL of failure
*/
const char *cmpack_all_get_filter(CmpackAllFile *file);

/**
	\brief Set heliocentric correction flag
	\details Set the flag to false if the julian dates are geocentric,
	set the flag to true if the julian dates are heliocentric.
	\param[in] file			file context
	\param[in] helcor		heliocentric correction flag
*/
void cmpack_all_set_helcor(CmpackAllFile *file, int helcor);

/**
	\brief Get heliocentric correction flag
	\details if the value is nonzero, julian dates are heliocentric, 
	otherwise, they are geocentric.
	\param[in] file			file context
	\return value of heliocentric correction flag
*/
int cmpack_all_get_helcor(CmpackAllFile *file);

/*************************   Frames   *****************************/

/**
	\brief Get number of frames defined in the file
	\param[in] file			file context
	\return number of frames or zero if the file
	is not valid context or if it is empty.
*/
int cmpack_all_nframes(CmpackAllFile *file);

/**
	\brief Get number of stars defined in the file
	\param[in] file			file context
	\return number of stars or zero if the file
	is not valid context or if it is empty.
*/
int cmpack_all_nstars(CmpackAllFile *file);

/**
	\brief Add new frame to the file.
	\param[in] file			file context
	\param[in] jd			date of observation
	\return frame index or negative value on failure.
*/
int cmpack_all_add_frame(CmpackAllFile *file, double jd);

/**	
	\brief Get date of observation
	\param[in] file			file context
	\param[in] frame		frame index
	\return julian date of observation or zero on failure
*/
double cmpack_all_get_jd(CmpackAllFile *file, int frame);

/**
	\brief Add measurement to the file
	\param[in] file			file context
	\param[in] frame		frame index
	\param[in] star			star index
	\param[in] mask			bitmask of valid fields
	\param[in] mag			magnitude
	\param[in] err			magnitude error
*/
void cmpack_all_set(CmpackAllFile *file, int frame, int star, 
		int mask, double mag, double err);

/**
	\brief Get measurement from the file
	\param[in] file			file context
	\param[in] frame		frame index
	\param[in] star			star index
	\param[out] mask		bitmask of valid fields
	\param[out] mag			magnitude
	\param[out] err			magnitude error
	\return zero on success, error code on failure
*/
int cmpack_all_get(CmpackAllFile *file, int frame, int star, 
		int *mask, double *mag, double *err);

/**
	\brief Make table of differential intrumental magnitudes
	\param[in] file			file context
	\param[in] var			index of variable star
	\param[in] comp			index of comparison star
	\param[out] tab			light curve
	\return zero on success, error code on failure
*/
int cmpack_all_lightcurve(CmpackAllFile *file, int var, int comp, 
		CmpackTable *tab);

#ifdef __cplusplus
}
#endif

#endif
