/**
	\file
	\brief Functions for the air-mass coefficient computation

	Set of functions defined in this module allows user to include
	the air-mass coefficients to a table stored in the file.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_amass.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/

#ifndef _CMPACK_AMASS_H_INCLUDED
#define _CMPACK_AMASS_H_INCLUDED

#include "cmpack_console.h"
#include "cmpack_table.h"

/********************   Private data structures   *****************************/

/**
	\brief Air-mass coefficient context
	\details This private data structure holds the parameter for air-mass
	coefficient computation.
*/
typedef struct _CmpackAirMass CmpackAirMass;

/**********************   Public constants   ***********************************/

/**
	\brief Correction flags
*/
typedef enum _CmpackAMassFlags
{
	CMPACK_AMASS_DEFAULT	= 0,		/**< Default behavior */
	CMPACK_AMASS_ALTITUDE	= (1<<0),	/**< Add altitude in degrees to a new column*/
	CMPACK_AMASS_NOAIRMASS	= (1<<1) 	/**< Do not add air mass coefficients */
} CmpackAMassFlags;

/********************   Public functions   *************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new air-mass coefficient computation context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure.
*/
CmpackAirMass *cmpack_amass_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			air-mass computation context
	\param[in] con			console context
*/
void cmpack_amass_set_console(CmpackAirMass *ctx, CmpackConsole *con);

/**
	\brief Set observer's longitude
	\param[in] ctx			air-mass computation context
	\param[in] lon			longitude in degrees (E+, W-)
*/
void cmpack_amass_set_lon(CmpackAirMass *ctx, double lon);

/**
	\brief Get observer's longitude
	\param[in] ctx			air-mass computation context
	\param[out] lon			longitude in degrees (E+, W-)
	\return zero on success or error code on failure
*/
int cmpack_amass_get_lon(CmpackAirMass *ctx, double *lon);

/**
	\brief Set observer's latitude
	\param[in] ctx			air-mass computation context
	\param[in] lat			latitude in degrees (N+, S-)
*/
void cmpack_amass_set_lat(CmpackAirMass *ctx, double lat);

/**
	\brief Get observer's latitude
	\param[in] ctx			air-mass computation context
	\param[out] lat			latitude in degrees (N+, S-)
	\return zero on success or error code on failure
*/
int cmpack_amass_get_lat(CmpackAirMass *ctx, double *lat);

/**
	\brief Set object's right ascension
	\param[in] ctx			air-mass computation context
	\param[in] ra			right ascension in hours
*/
void cmpack_amass_set_ra(CmpackAirMass *ctx, double ra);

/**
	\brief Get object's right ascension
	\param[in] ctx			air-mass computation context
	\param[out] ra			right ascension in hours
	\return zero on success or error code on failure
*/
int cmpack_amass_get_ra(CmpackAirMass *ctx, double *ra);

/**
	\brief Set object's declination
	\param[in] ctx			air-mass computation context
	\param[in] dec			declination in degrees
*/
void cmpack_amass_set_dec(CmpackAirMass *ctx, double dec);

/**
	\brief Get object's declination
	\param[in] ctx			air-mass computation context
	\param[out] dec			declination in degrees
	\return zero on success or error code on failure
*/
int cmpack_amass_get_dec(CmpackAirMass *ctx, double *dec);

/**
	\brief Include air-mass coefficients to a table stored in file
	\details The function read the input file line by line. All lines which started
	with value JD value are extended by corresponding air-mass coefficient. Other
	lines are copied into destination file without change.
	\param[in] ctx			air-mass computation context
	\param[in] table		table
	\param[in] column		column containing values of JD
	\param[in] flags		see CMPACK_AMASS_xxx constants
	\return zero on success or error code on failure.
*/
int cmpack_amass(CmpackAirMass *ctx, CmpackTable *table, int column, CmpackAMassFlags flags);

/**
	\brief Compute air-mass coefficient for single julian date.
	\param[in] ctx			air-mass computation context
	\param[in] jd			julian date of observation
	\param[out]	altitude	altitude in degrees above horizon
	\param[out] amass		air-mass coefficient
	\return zero on success or error code on failure
*/
int cmpack_amass_jd(CmpackAirMass *ctx, double jd, double *altitude, double *amass);

#ifdef __cplusplus
}
#endif

#endif
