/**
	\file
	\brief Functions for reading CCD frames

	Set of function defined in this module allows user to
	read CCD frame files in various formats by means of
	single interface. Writing files is not supported, use
	the cfitsio library directly.

	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_ccdfile.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_CCDFILE_H_INCLUDED
#define _CMPACK_CCDFILE_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"

/********************   Public constants   ********************************/

/** \brief Image format identifiers */
typedef enum _CmpackCcdFormat
{
	CMPACK_FORMAT_UNKNOWN = 0,		/**< Unknown format */
	CMPACK_FORMAT_FITS,				/**< FITS format */
	CMPACK_FORMAT_SBIG,				/**< SBIG format */
	CMPACK_FORMAT_OES				/**< OES Astro format */
} CmpackCcdFormat;

/** \brief Image transformation bits */
typedef enum _CmpackImageTrafo
{
	CMPACK_FLIP_0	= 0x00,			/**< No flipping */
	CMPACK_FLIP_X	= 0x01,			/**< Flip horizontally */
	CMPACK_FLIP_Y	= 0x02,			/**< Flip vertically */
	CMPACK_FLIP_XY	= 0x03			/**< Rotate image 180 degrees */
} CmpackImageTrafo;

/********************   Public data types   ********************************/

/**
	\brief CCD frame file context 
	\details This private structure is used to access CCD frame file
*/
typedef struct _CmpackCcdFile CmpackCcdFile;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Check if the file is a CCD frame
	\param[in] filename			path + file name
	\returns nonzero if the file is a CCD frame, zero otherwise
*/
int cmpack_ccd_test(const char *filename);

/**
	\brief Check if the content in buffer is a beginning of a CCD frame
	\param[in] buffer			beginning of the file
	\param[in] buflen			number of bytes stored in the buffer
	\param[in] filesize			size of file in bytes
	\returns nonzero if the file is a CCD frame, zero otherwise
*/
int cmpack_ccd_test_buffer(const char *buffer, int buflen, int filesize);

/**
	\brief Open a temporary CCD frame (stored in memory only)
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to new file context or zero on failure
*/
CmpackCcdFile *cmpack_ccd_new(void);

/**
	\brief Open a CCD frame file
	\param[out] fc				file context
	\param[in] filename			path + file name
	\param[in] flags			open flags
	\return zero on success, error code on failure
*/
int cmpack_ccd_open(CmpackCcdFile **fc, const char *filename, CmpackOpenMode flags);

/**
	\brief Make a copy of CCD frame
	\param[in] fc				target frame context
	\param[in] from			source frame context
	\return zero on success, error code on failure
*/
int cmpack_ccd_copy(CmpackCcdFile *fc, CmpackCcdFile *from);

/**
	\brief Get file format descriptor
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return file format descriptor or NULL on failure
*/
const char *cmpack_ccd_get_format(CmpackCcdFile *fc);

/**
	\brief Get date and time of observation
	\details The function reads the date and time at the start of the
	observation.
	\param[in] fc				file context
	\param[out] dt				date and time (zeros = not available)
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_datetime(CmpackCcdFile *fc, CmpackDateTime *dt);

/**
	\brief Get exposure duration
	\details The function reads exposure duration in seconds.
	\param[in] fc				file context
	\param[in] exptime			exposure duration
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_exptime(CmpackCcdFile *fc, double *exptime);

/**
	\brief Get CCD temperature
	\details The function reads the CCD temperature in degrees C.
	\param[in] fc				file context
	\param[in] ccdtemp			CCD temperature
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_ccdtemp(CmpackCcdFile *fc, double *ccdtemp);

/**
	\brief Get optical filter name
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return filter name or NULL on failure
*/
const char *cmpack_ccd_get_filter(CmpackCcdFile *fc);

/**
	\brief Get object's designation
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_object(CmpackCcdFile *fc);

/**
	\brief Get object's right ascension (R.A.)
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string are valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_objra(CmpackCcdFile *fc);

/**
	\brief Get object's declination
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_objdec(CmpackCcdFile *fc);

/**
	\brief Get observer's name
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_observer(CmpackCcdFile *fc);

/**
	\brief Get observatory name
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_observatory(CmpackCcdFile *fc);

/**
	\brief Get observer's longitude
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_obslon(CmpackCcdFile *fc);

/**
	\brief Get observer's latitude
	\details The function returns pointer to the internal memory. You must
	not free or modify it. The string is valid as far as the context exists.
	\param[in] fc				file context
	\return pointer to string or NULL on failure
*/
const char *cmpack_ccd_get_obslat(CmpackCcdFile *fc);

/**
	\brief Get number of frames combined together
	\details The function reads the number of frames. If the field is not defined,
	it returns zero.
	\param[in] fc				file context
	\param[in] frames			number of frames
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_frames(CmpackCcdFile *fc, int *frames);

/**
	\brief Parameter enumeration
	\details Index of the first parameter is zero. The output
	parameters receive the pointers to newly allocated memory buffers. 
	The caller is responsible to free it by calling of cmpack_free() 
	function. You can set the value of output parameter to NULL if you
	do not need it.
	\param[in] fc				file context
	\param[in] index			parameter index (starting by 0)
	\param[out] key				parameter name
	\param[out] val				parameter value
	\param[out] com				comment text
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_param(CmpackCcdFile *fc, int index, char **key, char **val, char **com);

/**
	\brief Get a string value from a file header
	\details The val parameter is set to a pointer to a newly allocated
	memory buffer. The caller is responsible to free it by calling of cmpack_free() 
	function. Keys are case sensitive.
	\param[in] fc				file context
	\param[in] key				parameter name (key)
	\param[out] val				parameter value
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_str(CmpackCcdFile *fc, const char *key, char **val);

/**
	\brief Get a integer value from a file header
	\param[in] fc				file context
	\param[in] key				parameter name (key)
	\param[out] val				parameter value
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_int(CmpackCcdFile *fc, const char *key, int *val);

/**
	\brief Get a logical value from a file header
	\param[in] fc				file context
	\param[in] key				parameter name (key)
	\param[out] val				parameter value
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_bool(CmpackCcdFile *fc, const char *key, int *val);

/**
	\brief Get a real value from a file header
	\param[in] fc				file context
	\param[in] key				parameter name (key)
	\param[out] val				parameter value
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_double(CmpackCcdFile *fc, const char *key, double *val);

/**
	\brief Get frame size in pixels
	\param[in] fc				file context
	\return image data type on success, zero on failure
*/
CmpackBitpix cmpack_ccd_get_bitpix(CmpackCcdFile *fc);

/**
	\brief Get image data range
	\param[in] fc				file context
	\param[in] minvalue			minimum pixel value
	\param[in] maxvalue			maximum pixel value
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_range(CmpackCcdFile *fc, double *minvalue, double *maxvalue);

/**
	\brief Get frame size in pixels
	\param[in] fc				file context
	\param[out] width			frame width in pixels (0 = not available)
	\param[out] height			frame height in pixels (0 = not available)
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_size(CmpackCcdFile *fc, int *width, int *height);

/**
	\brief Get image data (signed char)
	\details Image content is copied into given memory buffer.
	\param[in] fc				file context
	\param[in] bitpix			image resolution
	\param[in] flipx			horizonal image flipping
	\param[in] flipy			vertical image flipping
	\param[out] buf				image data
	\return zero on success, error code on failure
*/
int cmpack_ccd_get_image(CmpackCcdFile *fc, CmpackBitpix bitpix, int flipx, int flipy, void *buf);

/**
	\brief Print short info about specified CCD-frame
	\param[in] file			CCD-frame file path + name
	\param[in] con			console context
	\return zero on success or error code on failure
*/
int cmpack_ccd_print_info(CmpackCcdFile *file, CmpackConsole *con);

/**
	\brief Print full header from ccd frame of any format
	\param[in] file			CCD-frame file path + name
	\param[in] con			console context
	\return zero on success or error code on failure
*/
int cmpack_ccd_print_head(CmpackCcdFile *file, CmpackConsole *con);

#ifdef __cplusplus
}
#endif

#endif
