/**
	\file
	\brief Functions for the flat-frame correction

	Set of functions defined in this module allows user to 
	apply flat-frame correction to CCD frames.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_flat.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/

#ifndef _CMPACK_FLAT_H_INCLUDED
#define _CMPACK_FLAT_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Flat-frame correction context
	\details This private data structure holds the parameter for flat-frame
	correction as well as the flat frame itself.
*/
typedef struct _CmpackFlatCorr CmpackFlatCorr;

/********************   Public functions   *************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new flat-frame correction context.
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to the new flat frame context or zero on failure
*/
CmpackFlatCorr *cmpack_flat_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			flat correction context
	\param[in] con			console context
*/
void cmpack_flat_set_console(CmpackFlatCorr *ctx, CmpackConsole *con);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] ctx			flat correction context
	\param[in] border		border size in pixels
*/
void cmpack_flat_set_border(CmpackFlatCorr *ctx, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] ctx			flat correction context
	\param[out] border		border size in pixels
*/
void cmpack_flat_get_border(CmpackFlatCorr *ctx, CmpackBorder *border);

/**
	\brief Load flat-frame from a file
	\details The function reads the correction frame from the given frame 
	context. The internal copy of the image data is made, no reference to 
	the frame is held, so you can free it when this function has returned.
	\param[in] ctx			flat correction context
	\param[in] flatfile		bias frame context
	\return zero on success or error code on failure
*/
int cmpack_flat_rflat(CmpackFlatCorr *ctx, CmpackCcdFile *flatfile);

/**
	\brief Execute flat-frame correction 
	\details The function reads image data from the context, performs
	the flat correction and stores the output image to the same context.
	\param[in] ctx			flat correction context
	\param[in,out] file		frame context
	\return zero on success or error code on failure
*/
int cmpack_flat(CmpackFlatCorr *ctx, CmpackCcdFile *file);

/**
	\brief Execute flat-frame correction 
	\details The function reads image data from the infile context, performs
	the flat correction and stores the output image to the outfile context.
	\param[in] ctx			flat correction context
	\param[in] infile		input frame context
	\param[in] outfile		output frame context
	\return zero on success or error code on failure
*/
int cmpack_flat_ex(CmpackFlatCorr *ctx, CmpackCcdFile *infile, CmpackCcdFile *outfile);

#ifdef __cplusplus
}
#endif

#endif
