/**
	\file
	\brief Functions for matching photometry files.

	Set of functions defined in this module allows user to 
	find matching files, i.e. to find crossreferences in two sets
	of stars by their positions.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_match.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_MATCH_H_INCLUDED
#define _CMPACK_MATCH_H_INCLUDED

#include "cmpack_console.h"
#include "cmpack_phtfile.h"
#include "cmpack_catfile.h"

/********************   Data type definitions   ********************************/

/**
	\brief Matching configuration context
	\details This private data structure holds the configuration parameters for 
	CCD-frame conversion.
*/
typedef struct _CmpackMatch CmpackMatch;

/************************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new matching context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackMatch *cmpack_match_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			air-mass computation context
	\param[in] con			console context
*/
void cmpack_match_set_console(CmpackMatch *ctx, CmpackConsole *con);

/**
	\brief Set max. number of stars used for matching
	\param[in] ctx			matching context
	\param[in] nstars		number of stars
*/
void cmpack_match_set_maxstars(CmpackMatch *ctx, int nstars);

/**
	\brief Get max. number of stars used for matching
	\param[in] ctx			matching context
	\return number of stars
*/
int cmpack_match_get_maxstars(CmpackMatch *ctx);

/**
	\brief Set number of vertices of polygons
	\param[in] ctx			matching context
	\param[in] vertices		number of vertices
*/
void cmpack_match_set_vertices(CmpackMatch *ctx, int vertices);

/**
	\brief Get number of vertices of polygons
	\param[in] ctx			matching context
	\return number of vertices
*/
int cmpack_match_get_vertices(CmpackMatch *ctx);

/**
	\brief Set clipping threshold
	\param[in] ctx			matching context
	\param[in] threshold	threshold 
*/
void cmpack_match_set_threshold(CmpackMatch *ctx, double threshold);

/**
	\brief Get clipping threshold
	\param[in] ctx			matching context
	\return	current clipping threshold 
*/
double cmpack_match_get_threshold(CmpackMatch *ctx);

/**
	\brief Set matching method
	\param[in] ctx			matching context
	\param[in] method		(0=Standard, 1=Auto, 2=Sparse_fields)
*/
void cmpack_match_set_method(CmpackMatch *ctx, int method);

/**
	\brief Get matching method
	\param[in] ctx			matching context
	\return (0=Standard, 1=Auto, 2=Sparse_fields)
*/
int cmpack_match_get_method(CmpackMatch *ctx);

/**
	\brief Set Max. offset for 'sparse fields' method
	\param[in] ctx			matching context
	\param[in] maxoffset	offset in pixels
*/
void cmpack_match_set_maxoffset(CmpackMatch *ctx, double maxoffset);

/**
	\brief Get Max. offset for 'sparse fields' method
	\param[in] ctx			matching context
	\return offset in pixels
*/
double cmpack_match_get_maxoffset(CmpackMatch *ctx);

/**
	\brief Load reference frame from the photometry file
	\details The function reads photometry file into the context.
	\param[in] ctx			matching context
	\param[in] reffile		reference file context
	\return zero on success or error code on failure
*/
int cmpack_match_readref_pht(CmpackMatch *ctx, CmpackPhtFile *reffile);

/**
	\brief Load reference frame from the catalogue file
	\details The function reads catalogue file into the context.
	\param[in] ctx			matching context
	\param[in] reffile		reference file context
	\return zero on success or error code on failure
*/
int cmpack_match_readref_cat(CmpackMatch *ctx, CmpackCatFile *reffile);

/**
	\brief Match photometry file to the reference file
	\details The function reads the intput file, finds crossreferences to the
	reference frame and set the reference IDs.
	\param[in] ctx			matching context
	\param[in] file		photometry file context
	\param[out] mstars		number of stars matched
	\return zero on success or error code on failure
*/
int cmpack_match(CmpackMatch *ctx, CmpackPhtFile *file, int *mstars);

/**
	\brief Match photometry file to the reference file
	\details The function reads input file, finds crossreferences to the
	reference or catalogue file stored in the context and writes the
	output to another file.
	\param[in] ctx			matching context
	\param[in] infile		input file context
	\param[in] outfile		output file context
	\param[out] mstars		number of stars matched
	\return zero on success or error code on failure
*/

int cmpack_match_ex(CmpackMatch *ctx, CmpackPhtFile *infile, 
					CmpackPhtFile *outfile, int *mstars);

#ifdef __cplusplus
}
#endif

#endif
