/**
	\file
	\brief Functions for the making master-flat correction frame

	Set of functions defined in this module allows user to 
	make a master-flat frame from a set of CCD frames.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_mflat.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_MFLAT_H_INCLUDED
#define _CMPACK_MFLAT_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Master-flat output frame context
	\details This private data structure holds information needed to build
	one master-flat frame.
*/
typedef struct _CmpackMasterFlat CmpackMasterFlat;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new context for the Master-flat tool
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to the new master-flat context or zero on failure
*/
CmpackMasterFlat *cmpack_mflat_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			master-flat context
	\param[in] con			console context
*/
void cmpack_mflat_set_console(CmpackMasterFlat *ctx, CmpackConsole *con);

/**
	\brief Set output data format
	\param[in] ctx			master-flat context
	\param[in] bitpix		output data format
*/
void cmpack_mflat_set_bitpix(CmpackMasterFlat *ctx, CmpackBitpix bitpix);

/**
	\brief Get output data format
	\param[in] ctx			master-flat context
	\return output data format
*/
CmpackBitpix cmpack_mflat_get_bitpix(CmpackMasterFlat *ctx);

/**
	\brief Set average output level
	\param[in] ctx			master-flat context
	\param[in] level		mean level in ADU
*/
void cmpack_mflat_set_level(CmpackMasterFlat *ctx, double level);

/**
	\brief Get average output level
	\param[in] ctx			master-flat context
	\return mean level in ADU
*/
double cmpack_mflat_get_level(CmpackMasterFlat *ctx);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] ctx			master-flat context
	\param[in] border		border size in pixels
*/
void cmpack_mflat_set_border(CmpackMasterFlat *ctx, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] ctx			master-flat context
	\param[out] border		border size in pixels
*/
void cmpack_mflat_get_border(CmpackMasterFlat *ctx, CmpackBorder *border);

/**
	\brief Open output file
	\details Opens new master-flat frame. The caller is responsible to close the 
	file by calling the cmpack_mflat_close() function.
	\param[in] ctx			master-flat context
	\param[in] outfile		output file context
	\return zero on success or error code on failure.
*/
int cmpack_mflat_open(CmpackMasterFlat *ctx, CmpackCcdFile *outfile);

/**
	\brief Add a frame to the accumulation buffer
	\details The function reads the frame from specified CCD-frame 
	file into memory.
	\param[in] ctx			master-flat context
	\param[in] infile		input file context
	\return zero on success or error code on failure.
*/
int cmpack_mflat_read(CmpackMasterFlat *ctx, CmpackCcdFile *infile);

/**
	\brief Make master-flat frame and save it to the file
	\details The function computes the output CCD data, writes it
	to the file, closes the output file and frees allocated data.
	\param[in] ctx			master-flat context
	\return zero on success or error code on failure.
*/
int cmpack_mflat_close(CmpackMasterFlat *ctx);

#ifdef __cplusplus
}
#endif

#endif
