/**
	\file
	\brief Functions for making read-all files.

	Set of functions defined in this module allows user to 
	make read-all file from a set of photometry files.

	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_readall.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_READALL_H_INCLUDED
#define _CMPACK_READALL_H_INCLUDED

#include "cmpack_console.h"
#include "cmpack_allfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Read-all configuration context
	\details This private data structure holds the parameters for
	making read-all file.
*/
typedef struct _CmpackReadAll CmpackReadAll;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new configuration context for the Read-all tool
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackReadAll *cmpack_readall_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			air-mass computation context
	\param[in] con			console context
*/
void cmpack_readall_set_console(CmpackReadAll *ctx, CmpackConsole *con);

/**
	\brief Set aperture index
	\param[in] ctx			listing context
	\param[in] index		aperture index (starts by 1)
	\return zero on success or error code on failure
*/
int cmpack_readall_set_aperture(CmpackReadAll *ctx, int index);

/**
	\brief Get aperture index
	\param[in] ctx			listing context
	\param[out] index		aperture index (starts by 1)
	\return zero on success or error code on failure
*/
int cmpack_readall_get_aperture(CmpackReadAll *ctx, int *index);

/**
	\brief Open output read-all file
	\details Opens new read-all file and returns its context structure.
	The caller is responsible to close the file and free the allocated
	memory by calling the cmpack_readall_close() function.
	\param[in] ctx			configuration context
	\param[in] file			output file path + name
	\return zero on success or error code on failure.
*/
int cmpack_readall_open(CmpackReadAll *ctx, CmpackAllFile *file);

/**
	\brief Add a frame to the read-all file.
	\details The function reads data from specified photometry file into memory.
	\param[in] ctx			output file context
	\param[in] file			input file path + name
	\return zero on success or error code on failure.
*/
int cmpack_readall_read(CmpackReadAll *ctx, CmpackPhtFile *file);

/**
	\brief Close read-all file.
	\details The function closes the output file and frees allocated data.
	\param[in] ctx			output file context
	\return zero on success or error code on failure.
*/
int cmpack_readall_close(CmpackReadAll *ctx);

#ifdef __cplusplus
}
#endif

#endif
