/**
	\file
	\brief Functions for reading and writing output data files (tables).

	Output data files contain data in form of table. It is used for
	storing light curves, track lists, etc.

	Use cmpack-list object to process a set of photometry files and 
	make output data. This format was inherited from original Munipack,
	but it's been extended to support saving a set of parameters 
	(file header).
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_table.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_TABLE_H_INCLUDED
#define _CMPACK_TABLE_H_INCLUDED

#include "cmpack_common.h"

/*****************    Private data structures    ***************************/

/**
	\brief Table context
	\details This private structure holds the table header and data
*/
typedef struct _CmpackTable CmpackTable;

/**
	\brief Table column context
	\details This private structure holds the information about table column
*/
typedef struct _CmpackTabColumn CmpackTabColumn;

/** \brief Table type identifiers */
typedef enum _CmpackTableType
{
	CMPACK_TABLE_UNSPECIFIED,	/**< Unspecified type */
	CMPACK_TABLE_LIGHTCURVE,	/**< Light curve */
	CMPACK_TABLE_MUNIFIND,		/**< Mag-dev graph, result of the Munifind tool */
	CMPACK_TABLE_TRACKLIST,		/**< Track-list, frame offsets */
	CMPACK_TABLE_APERTURES,		/**< Apertures and standard deviations */
	CMPACK_TABLE_AIRMASS		/**< Air-mass coefficients */
} CmpackTableType;

/** \brief Column type identifier */
typedef enum _CmpackColumnType
{
	CMPACK_COLUMN_STRING,		/**< Unspecified type */
	CMPACK_COLUMN_INTEGER,		/**< Identifier of a star, an aperture, ... */
	CMPACK_COLUMN_DOUBLE		/**< Julian date */
} CmpackColumnType;

/********************   Public functions   *********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Create a new table context
	\details This function makes an empty table
	\return new reference to a table context
*/
CmpackTable *cmpack_tab_init(CmpackTableType type);

/**
	\brief Load a table from a file
	\param[out] tab			new table context
	\param[in] filename		path + file name
	\param[in] flags 		flags (see CMPACK_LOAD_xxx constants)
	\return zero on success, error code on failure
*/
int cmpack_tab_load(CmpackTable **tab, const char *filename, CmpackLoadMode flags);

/**
	\brief Save a table to a file
	\param[in] tab			table context
	\param[in] filename		path + file name
	\return zero on success, error code on failure
*/
int cmpack_tab_save(CmpackTable *tab, const char *filename);

/**
	\brief Clear table definition, header and data
	\details Deletes all information stored in a table context
	\param[in] tab			table context
*/
void cmpack_tab_clear(CmpackTable *tab);

/**
	\brief Copy a table
	\details Makes a copy of source table to the destination table.
	All content of the destination table will be deleted.
	\param[in] dsttab		destination table context
	\param[in] srctab		source table context
*/
void cmpack_tab_copy(CmpackTable *dsttab, const CmpackTable *srctab);

/**
	\brief Set table type
	\param[in] tab			table context
	\param[in] type			table type identifier
*/
void cmpack_tab_set_type(CmpackTable *tab, CmpackTableType type);

/**
	\brief Get table type
	\details This function returns the type of the table
	\return the integer value that represents the type of the table
*/
CmpackTableType cmpack_tab_get_type(CmpackTable *tab);

/**
	\brief Check if the file is a table
	\param[in] filename			path + file name
	\returns nonzero if the file is a table, zero otherwise
*/
int cmpack_tab_test(const char *filename);

/** 
	\brief Test if the content of the buffer might be a table
	\details The function checks the data in the buffer and returns nonzero,
	when the buffer contain first 'buflen' bytes from a table.
	This function is used for file format autodetection.
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_tab_test_buffer(const char *buffer, int buflen, int filesize);

/**********************   Table header   *****************************/

/**
	\brief Set parameter value in file header (string)
	\details If the parameter does not exist, it makes new entry in
	the file header. If the parameter exists, its content is changed.
	Keys are case sensitive.
	\param[in] tab			table context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\return zero on success, error code on failure
*/
void cmpack_tab_pkys(CmpackTable *tab, const char *key, const char *val);

/**
	\brief Set parameter value in file header (integer)
	\details If the parameter does not exist, it makes new entry in
	the file header. If the parameter exists, its content is changed. 
	Keys are case sensitive.
	\param[in] tab			table context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\return zero on success, error code on failure
*/
void cmpack_tab_pkyi(CmpackTable *tab, const char *key, int val);

/**
	\brief Set parameter value in file header (integer)
	\details If the parameter does not exist, it makes new entry in
	the file header. If the parameter exists, its content is changed. 
	Keys are case sensitive.
	\param[in] tab			table context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] prec			number of decimal places
	\return zero on success, error code on failure
*/
void cmpack_tab_pkyd(CmpackTable *tab, const char *key, double val, int prec);

/**
	\brief Get value of parameter in file header (string)
	\details The \a val parameters receives the pointer to newly allocated
	memory buffer. The caller is responsible to free it by calling of cmpack_free() 
	function. Keys are case sensitive. You can set the output parameter to NULL
	if you want to test the value only.
	\param[in] tab			table context
	\param[in] key			parameter name
	\return pointer to internal buffer or NULL on failure
*/
const char *cmpack_tab_gkys(CmpackTable *tab, const char *key);

/**
	\brief Get value of parameter in file header (integer)
	\details Keys are case sensitive.
	\param[in] tab			table context
	\param[in] key			parameter name
	\param[in] defval		default value
	\return parameter value on success, default value on failure
*/
int cmpack_tab_gkyi(CmpackTable *tab, const char *key, int defval);

/**
	\brief Get value of parameter in file header (double)
	\details Keys are case sensitive.
	\param[in] tab			table context
	\param[in] key			parameter name
	\param[in] defval		default value
	\return zero on success, error code on failure
*/
double cmpack_tab_gkyd(CmpackTable *tab, const char *key, double defval);

/**
	\brief Get number of parameters in file header
	\param[in] tab			table context
	\return number of parameter or zero if the file
	is not valid context or if the header is empty.
*/
int cmpack_tab_nkey(CmpackTable *tab);

/**
	\brief Get the parameter from file header by its index
	\details Index of the first parameter is zero. The output
	parameters receive the pointers to newly allocated memory buffers. 
	The caller is responsible to free it by calling of cmpack_free() 
	function. You can set the value of output parameter to NULL if you
	do not need to read it.
	\param[in] tab			table context
	\param[in] index		parameter index (starting by zero)
	\param[out] key			parameter name (can be NULL)
	\param[out] val			parameter value (can be NULL)
	\return zero on success, error code on failure
*/
int cmpack_tab_gkyn(CmpackTable *tab, int index, const char **key, const char **val);

/**
	\brief Delete parameter from file header
	\details Keys are case sensitive.
	\param[in] tab			table context
	\param[in] key			parameter name
*/
void cmpack_tab_dkey(CmpackTable *tab, const char *key);

/*******************   Table definition   ***************************/

/**
	\brief Get number of columns
	\param[in] tab			table context
	\return number of columns or zero if the table
	is not valid context or if it is empty.
*/
int cmpack_tab_ncolumns(CmpackTable *tab);

/**
	\brief Append new column to the table
	\param[in] tab			table context
	\param[in] name			column name
	\param[in] type			data type
	\param[in] prec			number of places after the decimal point
	\return index of new column or negative value on failure
*/
int cmpack_tab_add_column(CmpackTable *tab, const char *name, CmpackColumnType type, int prec); 

/**
	\brief Get column by its index
	\param[in] tab			table context
	\param[in] index		column index
	\return new reference to table column
*/
CmpackTabColumn *cmpack_tab_get_column(CmpackTable *tab, int index);

/**
	\brief Find column by its name
	\param[in] tab			table context
	\param[in] name			column name
	\return index of table column or negative value on failure
*/
int cmpack_tab_find_column(CmpackTable *tab, const char *name);

/**
	\brief Set column name 
	\details The function returns pointer to the internal structure.
	You must not call the cmpack_free() function to it.
	\param[in] col			column context
	\param[in] name			new column name
*/
void cmpack_tab_column_set_name(CmpackTabColumn *col, const char *name); 

/**
	\brief Get column name 
	\details The function returns pointer to the internal structure.
	You must not call the cmpack_free() function to it.
	\param[in] col			column context
	\return pointer to the column name
*/
const char *cmpack_tab_column_get_name(CmpackTabColumn *col); 

/**************************   Table data   ***************************/

/**
	\brief Get number of rows in the table
	\param[in] tab			table context
	\return If context is not valid or contains no data, returns zero
*/
int cmpack_tab_nrows(CmpackTable *tab);

/**
	\brief Append a new row to the table
	\details The function appends a new row to the end of the table
	\param[in] tab			table context
	\return index of new row or negative number on failure
*/
int cmpack_tab_add_row(CmpackTable *tab); 

/**
	\brief Set table data (string)
	\details The function modifies table data in specified row and column.
	\param[in] tab			table context
	\param[in] row			row index
	\param[in] col			column index
	\param[in] val			new value
	\return zero on success, error code on failure.
*/
void cmpack_tab_ptds(CmpackTable *tab, int row, int col, const char *val);

/**
	\brief Set table data (integer)
	\details The function modifies table data in specified row and column.
	\param[in] tab			table context
	\param[in] row			row index
	\param[in] col			column index
	\param[in] val			new value
	\return zero on success, error code on failure.
*/
void cmpack_tab_ptdi(CmpackTable *tab, int row, int col, int val);

/**
	\brief Set table data (real number)
	\details The function modifies table data in specified row and column.
	\param[in] tab			table context
	\param[in] row			row index
	\param[in] col			column index
	\param[in] val			new value
	\return zero on success, error code on failure.
*/
void cmpack_tab_ptdd(CmpackTable *tab, int row, int col, double val);

/**
	\brief Get table data (string)
	\param[in] tab			table context
	\param[in] row			row index
	\param[in] col			column index
	\param[out] val			pointer to internal buffer
	\return nonzero on success, zero on failure
*/
int cmpack_tab_gtds(CmpackTable *tab, int row, int col, char **val);

/**
	\brief Get table data (integer)
	\param[in] tab			table context
	\param[in] row			row index
	\param[in] col			column index
	\param[out] val			value
	\return nonzero on success, zero on failure
*/
int cmpack_tab_gtdi(CmpackTable *tab, int row, int col, int *val);

/**
	\brief Get table data (double)
	\param[in] tab			table context
	\param[in] row			row index
	\param[in] col			column index
	\param[out] val			value
	\return nonzero on success, zero on failure
*/
int cmpack_tab_gtdd(CmpackTable *tab, int row, int col, double *val);

#ifdef __cplusplus
}
#endif

#endif
